﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Threading;

namespace BackgroundWorker_MVVM
{
    /// <summary>
    /// The ProcessModel class implements the IEnumerable interface
    /// which allows us to use the object in a foreach loop.
    /// To simulate a long-running process, the enumerator contains
    /// a call to Sleep() in the MoveNext() method.
    /// </summary>
    /// <example>
    ///   var model = new ProcessModel(100);
    ///   foreach (var item  in model)
    ///   {
    ///     Console.WriteLine(item.ToString());
    ///   }
    /// </example>
    
    public class ProcessModel : IEnumerable<int>
    {
        /// <value>
        /// Determines how many values are returned when the
        /// object is enumerated (such as in a foreach loop).
        /// </value>
        public int Iterations { get; set; }

        public ProcessModel(int iterations)
        {
            Iterations = iterations;
        }

        #region IEnumerable Members

        IEnumerator IEnumerable.GetEnumerator()
        {
            return new ProcessEnum(Iterations);
        }

        IEnumerator<int> IEnumerable<int>.GetEnumerator()
        {
            return new ProcessEnum(Iterations);
        }

        #endregion

        #region IEnumerator

        private class ProcessEnum : IEnumerator<int>
        {
            private int _iterations;
            private int position = 0;

            public ProcessEnum(int iterations)
            {
                _iterations = iterations;
            }

            public bool MoveNext()
            {
                // Simulate a long-running process
                Thread.Sleep(100);
                position++;
                return (position <= _iterations);
            }

            public void Reset()
            {
                position = 0;
            }

            public int Current
            {
                get { return position; }
            }

            object IEnumerator.Current
            {
                get { return position; }
            }

            public void Dispose()
            {
                GC.SuppressFinalize(this);
            }
        }

        #endregion
    }
}
