﻿using System.ComponentModel;

namespace BackgroundWorker_MVVM
{
    public class ProcessViewModel : INotifyPropertyChanged
    {
        #region Fields

        private ProcessModel _model;
        private BackgroundWorker _worker;
        private int _iterations = 50;
        private int _progressPercentage = 0;
        private string _output;
        private bool _startEnabled = true;
        private bool _cancelEnabled = false;

        #endregion

        #region Bindable Properties

        public int Iterations
        {
            get { return _iterations; }
            set
            {
                if (_iterations != value)
                {
                    _iterations = value;
                    OnPropertyChanged(new PropertyChangedEventArgs("Iterations"));
                }
            }
        }

        public int ProgressPercentage
        {
            get { return _progressPercentage; }
            set
            {
                if (_progressPercentage != value)
                {
                    _progressPercentage = value;
                    OnPropertyChanged(new PropertyChangedEventArgs("ProgressPercentage"));
                }
            }
        }

        public string Output
        {
            get { return _output; }
            set
            {
                if (_output != value)
                {
                    _output = value;
                    OnPropertyChanged(new PropertyChangedEventArgs("Output"));
                }
            }
        }

        public bool StartEnabled
        {
            get { return _startEnabled; }
            set
            {
                if (_startEnabled != value)
                {
                    _startEnabled = value;
                    OnPropertyChanged(new PropertyChangedEventArgs("StartEnabled"));
                }
            }
        }

        public bool CancelEnabled
        {
            get { return _cancelEnabled; }
            set
            {
                if (_cancelEnabled != value)
                {
                    _cancelEnabled = value;
                    OnPropertyChanged(new PropertyChangedEventArgs("CancelEnabled"));
                }
            }
        }

        #endregion

        #region Constructor

        public ProcessViewModel()
        {
            _worker = new BackgroundWorker()
                {
                    WorkerReportsProgress = true,
                    WorkerSupportsCancellation = true
                };
            _worker.DoWork += worker_DoWork;
            _worker.ProgressChanged += worker_ProgressChanged;
            _worker.RunWorkerCompleted += worker_RunWorkerCompleted;
        }

        #endregion

        #region Public Methods

        public void StartProcess()
        {
            Output = "";
            if (!_worker.IsBusy)
            {
                if (_model == null)
                    _model = new ProcessModel(Iterations);
                else
                    _model.Iterations = Iterations;
                _worker.RunWorkerAsync(_model);
            }
            StartEnabled = !_worker.IsBusy;
            CancelEnabled = _worker.IsBusy;
        }

        public void CancelProcess()
        {
            _worker.CancelAsync();
        }

        #endregion

        #region BackgroundWorker Events

        // Note: This event fires on the background thread.
        private void worker_DoWork(object sender, DoWorkEventArgs e)
        {
            var worker = (BackgroundWorker)sender;
            var model = (ProcessModel)e.Argument;

            int result = 0;

            foreach (var val in model)
            {
                if (worker.CancellationPending)
                {
                    e.Cancel = true;
                    break;
                }
                if (worker.WorkerReportsProgress)
                {
                    int percentComplete = (int)((float)val / (float)model.Iterations * 100);
                    string updateMessage =
                        string.Format("Iteration {0} of {1}", val, model.Iterations);
                    worker.ReportProgress(percentComplete, updateMessage);
                }
                result = val;
            }
            e.Result = result;
        }

        // Note: This event fires on the UI thread.
        private void worker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                Output = e.Error.Message;
            }
            else if (e.Cancelled)
            {
                Output = "Cancelled";
            }
            else
            {
                Output = e.Result.ToString();
                ProgressPercentage = 0;
            }
            StartEnabled = !_worker.IsBusy;
            CancelEnabled = _worker.IsBusy;
        }

        // Note: This event fires on the UI thread.
        private void worker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            ProgressPercentage = e.ProgressPercentage;
            Output = (string)e.UserState;
        }

        #endregion

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;
        public void OnPropertyChanged(PropertyChangedEventArgs e)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, e);
        }

        #endregion
    }
}
