﻿using System;
using System.ComponentModel;
using System.Threading;
using System.Threading.Tasks;

namespace TaskWithProgress
{
    public class ProcessViewModel : INotifyPropertyChanged
    {
        #region Fields

        private ProcessModel _model;
        private CancellationTokenSource _cancelTokenSource;
        private int _iterations = 50;
        private int _progressPercentage = 0;
        private string _output;
        private bool _startEnabled = true;
        private bool _cancelEnabled = false;

        #endregion

        #region Bindable Properties

        public int Iterations
        {
            get { return _iterations; }
            set
            {
                if (_iterations != value)
                {
                    _iterations = value;
                    OnPropertyChanged(new PropertyChangedEventArgs("Iterations"));
                }
            }
        }

        public int ProgressPercentage
        {
            get { return _progressPercentage; }
            set
            {
                if (_progressPercentage != value)
                {
                    _progressPercentage = value;
                    OnPropertyChanged(new PropertyChangedEventArgs("ProgressPercentage"));
                }
            }
        }

        public string Output
        {
            get { return _output; }
            set
            {
                if (_output != value)
                {
                    _output = value;
                    OnPropertyChanged(new PropertyChangedEventArgs("Output"));
                }
            }
        }

        public bool StartEnabled
        {
            get { return _startEnabled; }
            set
            {
                if (_startEnabled != value)
                {
                    _startEnabled = value;
                    OnPropertyChanged(new PropertyChangedEventArgs("StartEnabled"));
                }
            }
        }

        public bool CancelEnabled
        {
            get { return _cancelEnabled; }
            set
            {
                if (_cancelEnabled != value)
                {
                    _cancelEnabled = value;
                    OnPropertyChanged(new PropertyChangedEventArgs("CancelEnabled"));
                }
            }
        }

        #endregion

        #region Constructor

        public ProcessViewModel()
        {

        }

        #endregion

        #region Public Methods

        public void StartProcess()
        {
            Output = "";
            StartEnabled = false;
            CancelEnabled = true;

            _cancelTokenSource = new CancellationTokenSource();

            if (_model == null)
                _model = new ProcessModel(Iterations);
            else
                _model.Iterations = Iterations;
            var task = DoWorkAsync(_model, _cancelTokenSource.Token,
                new Progress<ProgressObject>(UpdateProgress));
            task.ContinueWith(TaskComplete);
        }

        public void CancelProcess()
        {
            _cancelTokenSource.Cancel();
        }

        #endregion

        #region Task Methods

        public class ProgressObject
        {
            public int Percentage { get; set; }
            public string Message { get; set; }
        }

        private Task<int> DoWorkAsync(ProcessModel model, CancellationToken cancelToken,
            IProgress<ProgressObject> progress)
        {
            var task = Task<int>.Factory.StartNew(() =>
                {
                    int result = 0;
                    foreach (var val in model)
                    {
                        cancelToken.ThrowIfCancellationRequested();
                        int percentComplete = (int)((float)val / (float)model.Iterations * 100);
                        string updateMessage =
                            string.Format("Iteration {0} of {1}", val, model.Iterations);
                        progress.Report(new ProgressObject() 
                            { Percentage = percentComplete, Message = updateMessage });
                        result = val;
                    }
                    return result;
                },
                cancelToken);
            return task;
        }

        private void TaskComplete(Task<int> task)
        {
            if (task.IsFaulted)
            {
                // Handle exception state
            }
            else if (task.IsCanceled)
            {
                Output = "Canceled";
            }
            else if (task.IsCompleted)
            {
                Output = task.Result.ToString();
                ProgressPercentage = 0;
            }
            StartEnabled = true;
            CancelEnabled = false;
        }

        private void UpdateProgress(ProgressObject prog)
        {
            ProgressPercentage = prog.Percentage;
            Output = prog.Message;
        }

        #endregion

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;
        public void OnPropertyChanged(PropertyChangedEventArgs e)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, e);
        }

        #endregion
    }
}
